#include <adbm.h>
#include <stdio.h>
#include <assert.h>

#include "cmd.h"

/* R}hnh */
static int OnGetID( HCMD hCmd, void * pUserData ) ;
static int OnIsPathEnable( HCMD hCmd, void * pUserData ) ;
static int OnGetPath( HCMD hCmd, void * pUserData ) ;
static int OnIsHashEnable( HCMD hCmd, void * pUserData ) ;
static int OnIsImported( HCMD hCmd, void * pUserData ) ;
static int OnGetComment( HCMD hCmd, void * pUserData ) ;
static int OnSetComment( HCMD hCmd, void * pUserData ) ;
static int OnAttIDAdd( HCMD hCmd, void * pUserData ) ;
static int OnAttListup( HCMD hCmd, void * pUserData ) ;
static int OnAttIDRemove( HCMD hCmd, void * pUserData ) ;
static int OnAttIDRemoveAll( HCMD hCmd, void * pUserData ) ;
static int OnClose( HCMD hCmd, void * pUserData ) ;

/* R}h}bv */
static const CMDINFO c_cmdMap[] =
{
	{ "getid", OnGetID },
	{ "pathok", OnIsPathEnable },
	{ "getpath", OnGetPath },
	{ "hashok", OnIsHashEnable },
	{ "importok", OnIsImported },
	{ "getcomment", OnGetComment },
	{ "setcomment", OnSetComment },
	{ "attadd", OnAttIDAdd },
	{ "attlistup", OnAttListup },
	{ "attrm", OnAttIDRemove },
	{ "attrmall", OnAttIDRemoveAll },
	{ "close", OnClose },
} ;

/* fileinfo_main pf[^\ */
typedef struct tagFILEINFODATA
{
	ADBM_HADB		hADB ;
	ADBM_HFILEINFO	hFileInfo ;
} FILEINFODATA, * PFILEINFODATA ;

/* t@C񑀍 */
void fileinfo_main( ADBM_HADB hADB, ADBM_HFILEINFO hFileInfo )
{
	FILEINFODATA	data ;

	assert( hADB != NULL && hFileInfo != NULL ) ;
	data.hADB = hADB ;
	data.hFileInfo = hFileInfo ;
	cmd_loop( "fileinfo>", c_cmdMap, sizeof(c_cmdMap)/sizeof(CMDINFO), &data ) ;
}

/* R}hnh */
#ifndef UNUSED
#define UNUSED( x )	x
#endif
static int OnGetID( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;

	UNUSED( hCmd ) ;
	printf( "%d\n", fiGetID( pData->hFileInfo ) ) ;
	return 0 ;
}
static int OnIsPathEnable( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;

	UNUSED( hCmd ) ;
	if ( fiIsPathEnable( pData->hFileInfo ) )
		printf( "yes\n" ) ;
	else
		printf( "no\n" ) ;
	return 0 ;
}
static int OnGetPath( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;
	ADBM_CHAR		buffer[256] ;

	UNUSED( hCmd ) ;
	if ( fiGetPath( pData->hFileInfo, buffer, sizeof(buffer) ) )
		printf( "%s\n", buffer ) ;
	return 0 ;
}
static int OnIsHashEnable( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;

	UNUSED( hCmd ) ;
	if ( fiIsHashEnable( pData->hFileInfo ) )
		printf( "yes\n" ) ;
	else
		printf( "no\n" ) ;
	return 0 ;
}
static int OnIsImported( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;

	UNUSED( hCmd ) ;
	if ( fiIsImported( pData->hFileInfo ) )
		printf( "yes\n" ) ;
	else
		printf( "no\n" ) ;
	return 0 ;
}
static int OnGetComment( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;
	ADBM_CHAR		buffer[512] ;

	UNUSED( hCmd ) ;
	if ( fiGetComment( pData->hFileInfo, buffer, sizeof(buffer) ) )
		printf( "%s\n", buffer ) ;
	return 0 ;
}
static int OnSetComment( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;

	if ( !fiIsWriteAccess( pData->hFileInfo ) )
	{
		printf( "݃ANZX܂\n" ) ;
		return 0 ;
	}
	if ( cmd_argc( hCmd ) < 2 )
	{
		printf( "%s R}h\n", cmd_argv(hCmd,0) ) ;
		printf( "1:\tRg(K{)\n" ) ;
		return 0 ;
	}
	if ( !fiSetComment( pData->hFileInfo, cmd_argv(hCmd,1) ) )
		printf( "\n" ) ;
	else
		printf( "s\n" ) ;
	return 0 ;
}
static int OnAttIDAdd( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;
	ADBM_UINT32		attID = 0 ;

	if ( !fiIsWriteAccess( pData->hFileInfo ) )
	{
		printf( "݃ANZX܂\n" ) ;
		return 0 ;
	}
	if ( cmd_argc( hCmd ) < 2 )
	{
		printf( "%s R}h\n", cmd_argv(hCmd,0) ) ;
		printf( "1:\t(K{)\n" ) ;
		return 0 ;
	}
	adbAttInfoAdd( pData->hADB, cmd_argv(hCmd,1), &attID ) ;
	if ( attID == 0 )
	{
		printf( "s\n" ) ;
		return 0 ;
	}
	if ( !fiAttIDAdd( pData->hFileInfo, attID ) )
		printf( "\n" ) ;
	else
		printf( "s\n" ) ;
	return 0 ;
}
static int OnAttListup( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;
	ADBM_UINT32		attNum, i ;

	UNUSED( hCmd ) ;
	attNum = fiAttIDGetNum( pData->hFileInfo ) ;
	printf( "tF%d\n", attNum ) ;
	for ( i = 0 ; i < attNum ; i ++ )
	{
		ADBM_HATTINFO	hAttInfo = adbAttInfoOpen( pData->hADB, fiAttIDGetAt(pData->hFileInfo,i), ADBM_FALSE ) ;
		if ( hAttInfo != NULL )
		{
			char	attName[256] ;
			if ( aiGetName( hAttInfo, attName, sizeof(attName) ) > 0 )
				printf( "%dF%s(ID %d)\n", i, attName, aiGetID(hAttInfo) ) ;
			adbAttInfoClose( pData->hADB, hAttInfo ) ;
		}
	}
	return 0 ;
}
static int OnAttIDRemove( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;
	ADBM_UINT32		attID, index ;

	UNUSED( hCmd ) ;
	if ( !fiIsWriteAccess( pData->hFileInfo ) )
	{
		printf( "݃ANZX܂\n" ) ;
		return 0 ;
	}
	if ( cmd_argc( hCmd ) < 2 )
	{
		printf( "%s R}h\n", cmd_argv(hCmd,0) ) ;
		printf( "1:\t(K{)\n" ) ;
		return 0 ;
	}
	attID = adbAttInfoGetID( pData->hADB, cmd_argv(hCmd,1) ) ;
	if ( attID == 0 )
	{
		printf( "w肳ꂽ͓o^Ă܂\n" ) ;
		return 0 ;
	}
	if ( fiAttIDFind( pData->hFileInfo, attID, &index ) )
	{
		if ( !fiAttIDRemoveAt( pData->hFileInfo, index ) )
			printf( "\n" ) ;
		else
			printf( "s\n" ) ;
	}
	else
	{
		printf( "w肳ꂽ͕tĂ܂\n" ) ;
		return 0 ;
	}
	return 0 ;
}
static int OnAttIDRemoveAll( HCMD hCmd, void * pUserData )
{
	PFILEINFODATA	pData = (PFILEINFODATA)pUserData ;

	UNUSED( hCmd ) ;
	if ( !fiIsWriteAccess( pData->hFileInfo ) )
	{
		printf( "݃ANZX܂\n" ) ;
		return 0 ;
	}
	if ( !fiAttIDRemoveAll( pData->hFileInfo ) )
		printf( "\n" ) ;
	else
		printf( "s\n" ) ;
	return 0 ;
}
static int OnClose( HCMD hCmd, void * pUserData )
{
	UNUSED( hCmd ) ;
	UNUSED( pUserData ) ;
	return -1 ;
}
